<?php

namespace Drupal\Tests\commerce_merchant_warrior\Unit\Api;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\commerce_merchant_warrior\Api\MerchantWarriorApi;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Psr7\Response;
use PHPUnit\Framework\TestCase;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * @coversDefaultClass \Drupal\commerce_merchant_warrior\Api\MerchantWarriorApi
 *
 * Tests the MerchantWarriorApi class.
 */
class MerchantWarriorApiTest extends TestCase {

  /**
   * The config factory mock.
   *
   * @var \PHPUnit\Framework\MockObject\MockObject|\Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The config mock.
   *
   * @var \PHPUnit\Framework\MockObject\MockObject|\Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The HTTP client mock.
   *
   * @var \PHPUnit\Framework\MockObject\MockObject|\GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The logger mock.
   *
   * @var \PHPUnit\Framework\MockObject\MockObject|\Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The request stack mock.
   *
   * @var \PHPUnit\Framework\MockObject\MockObject|\Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The MerchantWarriorApi instance.
   *
   * @var \Drupal\commerce_merchant_warrior\Api\MerchantWarriorApi
   */
  protected $api;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    $this->configFactory = $this->createMock(ConfigFactoryInterface::class);
    $this->config = $this->createMock(ImmutableConfig::class);
    $this->httpClient = $this->createMock(ClientInterface::class);
    $this->logger = $this->createMock(LoggerInterface::class);
    $this->requestStack = $this->createMock(RequestStack::class);

    $this->configFactory->expects($this->any())
      ->method('get')
      ->with('commerce_payment.commerce_payment_gateway.merchant_warrior_payframe')
      ->willReturn($this->config);

    $this->api = new MerchantWarriorApi($this->configFactory, $this->httpClient, $this->logger, $this->requestStack);
  }

  /**
   * Data provider for testVerifyCard.
   */
  public function verifyCardDataProvider(): array {
    return [
      'success' => [
        'response' => '<response><responseCode>0</responseCode><responseMessage>Success</responseMessage></response>',
        'expected_exception' => NULL,
        'expected_message' => NULL,
      ],
      'failure' => [
        'response' => '<response><responseCode>1</responseCode><responseMessage>Error message</responseMessage></response>',
        'expected_exception' => PaymentGatewayException::class,
        'expected_message' => 'Merchant Warrior: Unable to verify the payment method. The error response was: Error message.',
      ],
    ];
  }

  /**
   * Test the verifyCard method.
   *
   * @dataProvider verifyCardDataProvider
   */
  public function testVerifyCard($response, $expected_exception, $expected_message): void {
    $this->config->method('get')->willReturn([
      'merchant_uuid' => 'test_uuid',
      'api_key' => 'test_key',
      'api_passphrase' => 'test_passphrase',
      'mode' => 'test',
    ]);

    $this->httpClient->method('request')->willReturn(new Response(200, [], $response));

    $transaction_data = [
      'transactionAmount' => '10.00',
      'transactionCurrency' => 'AUD',
      'transactionProduct' => 'Test Product',
      'paymentCardNumber' => '5123456789012346',
      'paymentCardExpiry' => '0139',
      'paymentCardName' => 'Bob Jones',
      'paymentCardCSC' => '123',
    ];

    if ($expected_exception) {
      $this->expectException($expected_exception);
      $this->expectExceptionMessage($expected_message);
    }

    $result = $this->api->verifyCard($transaction_data);

    if (!$expected_exception) {
      $this->assertIsArray($result);
      $this->assertEquals('0', $result['responseCode']);
      $this->assertEquals('Success', $result['responseMessage']);
    }
  }

  /**
   * Data provider for testCardInfo.
   */
  public function cardInfoDataProvider(): array {
    return [
      'success' => [
        'response' => '<response><responseCode>0</responseCode><responseMessage>Success</responseMessage></response>',
        'expected_exception' => NULL,
        'expected_message' => NULL,
      ],
      'failure' => [
        'response' => '<response><responseCode>1</responseCode><responseMessage>Error message</responseMessage></response>',
        'expected_exception' => PaymentGatewayException::class,
        'expected_message' => 'Merchant Warrior: Unable to fetch additional card details for the payment method. The error response was: Error message.',
      ],
    ];
  }

  /**
   * Test the cardInfo method.
   *
   * @dataProvider cardInfoDataProvider
   */
  public function testCardInfo($response, $expected_exception, $expected_message): void {
    $this->config->method('get')->willReturn([
      'merchant_uuid' => 'test_uuid',
      'api_key' => 'test_key',
      'api_passphrase' => 'test_passphrase',
      'mode' => 'test',
    ]);

    $this->httpClient->method('request')->willReturn(new Response(200, [], $response));

    $transaction_data = [
      'cardID' => '12345',
    ];

    if ($expected_exception) {
      $this->expectException($expected_exception);
      $this->expectExceptionMessage($expected_message);
    }

    $result = $this->api->cardInfo($transaction_data);

    if (!$expected_exception) {
      $this->assertIsArray($result);
      $this->assertEquals('0', $result['responseCode']);
      $this->assertEquals('Success', $result['responseMessage']);
    }
  }

  /**
   * Data provider for testGetAccessToken.
   */
  public function getAccessTokenDataProvider(): array {
    return [
      'success' => [
        'response' => '<response><responseCode>0</responseCode><token>test_token</token></response>',
        'expected_exception' => NULL,
        'expected_message' => NULL,
        'expected_result' => 'test_token',
      ],
      'failure' => [
        'response' => '<response><responseCode>1</responseCode><responseMessage>Error message</responseMessage></response>',
        'expected_exception' => PaymentGatewayException::class,
        'expected_message' => 'Merchant Warrior: Unable to fetch an access token from Merchant Warrior. The error response was: Error message.',
        'expected_result' => NULL,
      ],
    ];
  }

  /**
   * Test the getAccessToken method.
   *
   * @dataProvider getAccessTokenDataProvider
   */
  public function testGetAccessToken($response, $expected_exception, $expected_message, $expected_result): void {
    $this->config->method('get')->willReturn([
      'merchant_uuid' => 'test_uuid',
      'api_key' => 'test_key',
      'api_passphrase' => 'test_passphrase',
      'mode' => 'test',
    ]);

    $this->httpClient->method('request')->willReturn(new Response(200, [], $response));

    if ($expected_exception) {
      $this->expectException($expected_exception);
      $this->expectExceptionMessage($expected_message);
    }

    $result = $this->api->getAccessToken();

    if (!$expected_exception) {
      $this->assertEquals($expected_result, $result);
    }
  }

  /**
   * Data provider for testProcessAuthorization.
   */
  public function processAuthorizationDataProvider(): array {
    return [
      'success' => [
        'response' => '<response><responseCode>0</responseCode><responseMessage>Success</responseMessage></response>',
        'expected_exception' => NULL,
        'expected_message' => NULL,
      ],
      'failure' => [
        'response' => '<response><responseCode>1</responseCode><responseMessage>Error message</responseMessage></response>',
        'expected_exception' => PaymentGatewayException::class,
        'expected_message' => 'Merchant Warrior: Unable to authorize the payment for Order ID: 12345. The error response was: Error message.',
      ],
    ];
  }

  /**
   * Test the processAuthorization method.
   *
   * @dataProvider processAuthorizationDataProvider
   */
  public function testProcessAuthorization($response, $expected_exception, $expected_message): void {
    $this->config->method('get')->willReturn([
      'merchant_uuid' => 'test_uuid',
      'api_key' => 'test_key',
      'api_passphrase' => 'test_passphrase',
      'mode' => 'test',
    ]);

    $this->httpClient->method('request')->willReturn(new Response(200, [], $response));

    $transaction_data = [
      'transactionAmount' => '10.00',
      'transactionCurrency' => 'AUD',
      'order_id' => '12345',
    ];

    if ($expected_exception) {
      $this->expectException($expected_exception);
      $this->expectExceptionMessage($expected_message);
    }

    $result = $this->api->processAuthorization($transaction_data);

    if (!$expected_exception) {
      $this->assertIsArray($result);
      $this->assertEquals('0', $result['responseCode']);
      $this->assertEquals('Success', $result['responseMessage']);
    }
  }

  /**
   * Data provider for testProcessCapture.
   */
  public function processCaptureDataProvider(): array {
    return [
      'success' => [
        'response' => '<response><responseCode>0</responseCode><responseMessage>Success</responseMessage></response>',
        'expected_exception' => NULL,
        'expected_message' => NULL,
      ],
      'failure' => [
        'response' => '<response><responseCode>1</responseCode><responseMessage>Error message</responseMessage></response>',
        'expected_exception' => PaymentGatewayException::class,
        'expected_message' => 'Merchant Warrior: Unable to capture the payment for Order ID: 12345. The error response was: Error message.',
      ],
    ];
  }

  /**
   * Test the processCapture method.
   *
   * @dataProvider processCaptureDataProvider
   */
  public function testProcessCapture($response, $expected_exception, $expected_message): void {
    $this->config->method('get')->willReturn([
      'merchant_uuid' => 'test_uuid',
      'api_key' => 'test_key',
      'api_passphrase' => 'test_passphrase',
      'mode' => 'test',
    ]);

    $this->httpClient->method('request')->willReturn(new Response(200, [], $response));

    $transaction_data = [
      'transactionAmount' => '10.00',
      'transactionCurrency' => 'AUD',
      'order_id' => '12345',
    ];

    if ($expected_exception) {
      $this->expectException($expected_exception);
      $this->expectExceptionMessage($expected_message);
    }

    $result = $this->api->processCapture($transaction_data);

    if (!$expected_exception) {
      $this->assertIsArray($result);
      $this->assertEquals('0', $result['responseCode']);
      $this->assertEquals('Success', $result['responseMessage']);
    }
  }

  /**
   * Data provider for testProcessRefund.
   */
  public function processRefundDataProvider(): array {
    return [
      'success' => [
        'response' => '<response><responseCode>0</responseCode><responseMessage>Success</responseMessage></response>',
        'expected_exception' => NULL,
        'expected_message' => NULL,
      ],
      'failure' => [
        'response' => '<response><responseCode>1</responseCode><responseMessage>Error message</responseMessage></response>',
        'expected_exception' => PaymentGatewayException::class,
        'expected_message' => 'Merchant Warrior: Unable to refund the payment for Order ID: 12345. The error response was: Error message.',
      ],
    ];
  }

  /**
   * Test the refundCard method.
   *
   * @dataProvider processRefundDataProvider
   */
  public function testProcessRefund($response, $expected_exception, $expected_message): void {
    $this->config->method('get')->willReturn([
      'merchant_uuid' => 'test_uuid',
      'api_key' => 'test_key',
      'api_passphrase' => 'test_passphrase',
      'mode' => 'test',
    ]);

    $this->httpClient->method('request')->willReturn(new Response(200, [], $response));

    $transaction_data = [
      'transactionAmount' => '10.00',
      'transactionCurrency' => 'AUD',
      'order_id' => '12345',
    ];

    if ($expected_exception) {
      $this->expectException($expected_exception);
      $this->expectExceptionMessage($expected_message);
    }

    $result = $this->api->refundCard($transaction_data);

    if (!$expected_exception) {
      $this->assertIsArray($result);
      $this->assertEquals('0', $result['responseCode']);
      $this->assertEquals('Success', $result['responseMessage']);
    }
  }

}
